<?php
/**
 * "Conditional Redirect" ACFE Form Action Class.
 *
 * Handles the "Conditional Redirect" ACFE Form Action.
 *
 * @package Conditional_Form_Actions_For_ACFE
 */

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * "Conditional Redirect" ACFE Form Action Class.
 *
 * A class that handles the "Conditional Redirect" ACFE Form Action.
 *
 * @since 0.2.0
 */
class CFAFA_ACFE_Form_Action_Redirect extends CFAFA_ACFE_Form_Action_Base {

	/**
	 * Plugin object.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var Conditional_Form_Actions_For_ACFE
	 */
	public $plugin;

	/**
	 * ACF Extended object.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var CFAFA_ACFE
	 */
	public $acfe;

	/**
	 * Form Action Name.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $name = 'redirect_cfafa';

	/**
	 * Form Action Name.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $action_name = 'redirect_cfafa';

	/**
	 * Field Key Prefix.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $field_key = 'field_cfafa_redirect_';

	/**
	 * Field Name Prefix.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $field_name = 'cfafa_redirect_';

	/**
	 * Constructor.
	 *
	 * @since 0.2.0
	 */
	public function __construct() {

		// Store references to objects.
		$this->plugin = cfafa();
		$this->acfe   = $this->plugin->acfe;

		// Label this Form Action.
		$this->title = __( 'Conditional Redirect action', 'conditional-form-actions-for-acfe' );

		// Alias Placeholder for this Form Action.
		$this->alias_placeholder = __( 'Conditional Redirect', 'conditional-form-actions-for-acfe' );

		// Conditional Code for this Form Action.
		$this->conditional_code = 'redirect_conditional';

		// Declare core Fields for this Form Action.
		$this->item = [
			'action' => $this->name,
			'name'   => '',
		];

		// Declare Fields for this Form Action.
		$this->item[ $this->field_name . 'redirect_url' ] = '';

		// Declare Conditional Field for this Form Action.
		$this->item[ $this->field_name . 'map_' . $this->conditional_code ] = '';

		// Init parent.
		parent::__construct();

	}

	/**
	 * Performs the action when the Form the Action is attached to is submitted.
	 *
	 * @since 0.2.0
	 *
	 * @param array $form The array of Form data.
	 * @param array $action The array of Action data.
	 */
	public function make_action( $form, $action ) {

		// Bail if a filter has overridden the action.
		if ( false === $this->make_skip( $form, $action ) ) {
			return;
		}

		// Populate Conditional Reference and value.
		$action = $this->form_conditional_populate( $action );

		// Populate Redirect data array.
		$redirect = $this->form_redirect_data( $form, $action );

		// Save the results of this Action for later use.
		$this->make_action_save( $action, $redirect );

		// Check Conditional.
		if ( ! $this->form_conditional_check( $action ) ) {
			return;
		}

		// Strip out any empty Fields.
		$redirect_data = $this->form_data_prepare( $redirect );

		// Sanity check.
		if ( empty( $redirect_data['redirect_url'] ) ) {
			return;
		}

		/**
		 * Fires the ACF Extended 0.9.x success action.
		 *
		 * This is to allow any plugins that perform administrative tasks (e.g. saving
		 * form submissions) to act before the redirect.
		 *
		 * Note that `acfe/form/submit_form` cannot be fired here because doing so
		 * causes recursive calls to `make_action()`.
		 *
		 * @see https://develop.tadpole.cc/plugins/submission-logs-for-acfe
		 *
		 * @since 0.2.0
		 *
		 * @param array $form The array of Form data.
		 */
		do_action( 'acfe/form/submit_success', $form );

		// Do the redirect.
		wp_safe_redirect( $redirect_data['redirect_url'] );
		exit;

	}

	/**
	 * Defines additional Fields for the "Action" Tab.
	 *
	 * @since 0.2.0
	 *
	 * @return array $fields The array of Fields for this section.
	 */
	public function tab_action_append() {

		// Init Fields.
		$fields = [];

		// Add URL Field.
		$fields[] = [
			'key'               => $this->field_key . 'redirect_url',
			'label'             => __( 'Redirect URL', 'conditional-form-actions-for-acfe' ),
			'name'              => $this->field_name . 'redirect_url',
			'type'              => 'text',
			'instructions'      => __( 'The URL to redirect to. See "Cheatsheet" tab for all available template tags.', 'conditional-form-actions-for-acfe' ),
			'required'          => 0,
			'conditional_logic' => 0,
			'wrapper'           => [
				'width'                      => '',
				'class'                      => '',
				'id'                         => '',
				'data-instruction-placement' => 'field',
			],
			'acfe_permissions'  => '',
			'default_value'     => '',
			'placeholder'       => '',
			'prepend'           => '',
			'append'            => '',
			'maxlength'         => '',
		];

		// Add Conditional Field.
		$label                      = __( 'Conditional On', 'conditional-form-actions-for-acfe' );
		$conditional                = $this->mapping_field_get( $this->conditional_code, $label );
		$conditional['placeholder'] = __( 'Always redirect', 'conditional-form-actions-for-acfe' );
		$conditional['wrapper']['data-instruction-placement'] = 'field';
		$conditional['instructions']                          = __( 'To redirect only when a Form Field is populated, link this to the Form Field. To redirect only when more complex conditions are met, link this to a Hidden Field with value "1" where the conditional logic of that Field shows it when the conditions are met.', 'conditional-form-actions-for-acfe' );
		$fields[] = $conditional;

		// --<
		return $fields;

	}

	/**
	 * Builds Redirect data array from mapped Fields.
	 *
	 * @since 0.2.0
	 *
	 * @param array $form The array of Form data.
	 * @param array $action The array of Action data.
	 * @return array $data The array of results of this Action to save for later use.
	 */
	public function form_redirect_data( $form, $action ) {

		// Init data array.
		$data = [];

		// Get Form name.
		$form_name = acf_maybe_get( $form, 'name' );

		// Set ACFE "context". We want unformatted data.
		$context = [
			'context' => 'save',
			'format'  => false,
		];

		// Get the Action variables.
		acfe_apply_tags( $action[ $this->field_name . 'redirect_url' ], $context );
		$url = $action[ $this->field_name . 'redirect_url' ];

		/**
		 * Filter the Redirect URL.
		 *
		 * @since 0.2.0
		 *
		 * @param string $url The Redirect URL.
		 * @param array  $form The array of Form data.
		 * @param array  $action The array of Action data.
		 */
		$filter = 'acfe/form/v3/submit/' . $this->action_name . '/url';
		$url    = apply_filters( $filter, $url, $form, $action );
		$url    = apply_filters( $filter . '/form=' . $form_name, $url, $form, $action );
		if ( ! empty( $action['name'] ) ) {
			$url = apply_filters( $filter . '/action=' . $action['name'], $url, $form, $action );
		}

		// Add trimmed URL to the data array.
		$data['redirect_url'] = trim( $url );

		// Also save conditional.
		$data['conditional'] = $action['conditional'];

		// --<
		return $data;

	}

}
